package com.mortals.xhx.module.child.web;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.annotation.DataPermission;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.module.certificate.model.CertificateDocumentQuery;
import com.mortals.xhx.module.certificate.service.CertificateDocumentService;
import com.mortals.xhx.module.device.model.DeviceEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.child.model.ChildLicenseEntity;
import com.mortals.xhx.module.child.service.ChildLicenseService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

import com.mortals.xhx.common.code.*;

/**
 * 行业许可子证
 *
 * @author zxfei
 * @date 2024-07-28
 */
@RestController
@RequestMapping("child/license")
public class ChildLicenseController extends BaseCRUDJsonBodyMappingController<ChildLicenseService, ChildLicenseEntity, Long> {


    @Autowired
    private CertificateDocumentService certificateDocumentService;

    public ChildLicenseController() {
        super.setModuleDesc("行业许可子证");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "legalPersonName", LegalPersonNameEnum.getEnumMap());
        this.addDict(model, "processStatus", ProcessStatusEnum.getEnumMap());
        this.addDict(model, "childStatus", ChildStatusEnum.getEnumMap());
        this.addDict(model, "deleted", DeletedEnum.getEnumMap());
        this.addDict(model, "documentId", certificateDocumentService.find(new CertificateDocumentQuery())
                .stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDocumentName(), (o, n) -> n)));
        super.init(model, context);
    }

    @Override
    @DataPermission
    public Rest<Object> list(@RequestBody ChildLicenseEntity query) {
        return super.list(query);
    }

    @Override
    protected void doListBefore(ChildLicenseEntity query, Map<String, Object> model, Context context) throws AppException {

        if(!ObjectUtils.isEmpty(context.getUser())){
            Long deptId = context.getUser().getDeptId();
            if(!context.getUser().isAdmin()){
                query.setDeptId(deptId);
            }
        }

        super.doListBefore(query, model, context);
    }

    @Override
    protected void doListAfter(ChildLicenseEntity query, List<ChildLicenseEntity> list, Context context) throws AppException {
        list.forEach(item -> {
            item.setFormContent("");
            item.setFormStyleContent("");
        });

        super.doListAfter(query, list, context);
    }


    /**
     * 子证联报
     */
    @PostMapping(value = "report")
    public String report(@RequestBody ChildLicenseEntity childLicenseEntity) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = this.getModuleDesc() + "联报";
        try {
            if (childLicenseEntity.newEntity()) throw new AppException("Id不能为空！");
            ChildLicenseEntity sourceEntity = this.service.get(childLicenseEntity.getId());
            if (ProcessStatusEnum.已处理.getValue() == sourceEntity.getProcessStatus())
                throw new AppException("当前子证已经联报");

            childLicenseEntity.setReportUserId(this.getContextUserId(getContext()));
            childLicenseEntity.setReportUserName(this.getContext().getUser().getRealName());
            childLicenseEntity.setReportTime(new Date());
            childLicenseEntity.setProcessStatus(ProcessStatusEnum.已处理.getValue());


            this.service.update(childLicenseEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        } catch (Exception e) {
            log.error("子证联报异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }
}